from typing import Optional

from arroyo.processing.strategies.abstract import ProcessingStrategy
from arroyo.types import Commit, Message, TPayload


class CommitOffsets(ProcessingStrategy[TPayload]):
    """
    Just commits offsets.

    This should always be used as the last step in a chain of processing
    strategies. It commits offsets back to the broker after all prior
    processing of that message is completed.
    """

    def __init__(self, commit: Commit) -> None:
        self.__commit = commit

    def poll(self) -> None:
        pass

    def submit(self, message: Message[TPayload]) -> None:
        self.__commit({message.partition: message.position_to_commit})

    def close(self) -> None:
        pass

    def terminate(self) -> None:
        pass

    def join(self, timeout: Optional[float] = None) -> None:
        # Commit all previously staged offsets
        self.__commit({}, force=True)
